import 'dart:collection';

import 'package:disposable_utils/src/disposable.dart';
import 'package:meta/meta.dart';

/// Base class for [Disposable]`s that contains many other [Disposable]s,
/// which you can dispose as one.
class DisposableCollector extends ListBase<Disposable> implements Disposable {
  List<Disposable> _disposables = [];

  @override
  @mustCallSuper
  /// Disposes all [Disposable]s it contains
  void dispose() => _disposables.forEach((d) => d.dispose());

  @override void add(Disposable element) => _disposables.add(element);
  @override int get length => _disposables.length;
  @override set length(value) => _disposables.length = value;
  @override Disposable operator [](int index) => _disposables[index];
  @override void operator []=(int index, Disposable value) => _disposables[index] = value;
}